@@ -14,14 +14,14 @@ pub fn run_compiler_frontend<'a>(source: &'a str) -> Vec<crate::backend::Ast<'a>
     let children = passes::to_unnormalized_backend_ir(children);
     // NORMALIZE SETUP
     let transfomer = backend::ast::ChildListTransformer {
-        parameters: Rc::new(std::convert::identity),
-        block: Rc::new(passes::normalize_ir),
+        parameters: Rc::new(passes::parameter_level_normalize),
+        block: Rc::new(passes::block_level_normalize),
         rewrite_rules: Rc::new(std::convert::identity),
         marker: std::marker::PhantomData
     };
-    let node = backend::Ast::new_fragment(children);
     // NORMALIZE BACKNED IR
-    let node = node.child_list_transformer(Rc::new(transfomer));
+    let node = backend::Ast::new_fragment(children)
+        .child_list_transformer(Rc::new(transfomer));
     // DONE
     node.into_fragment()
 }
